import 'dart:convert';
import 'package:dartz/dartz.dart';
import 'package:yaml/yaml.dart';
// import 'package:flutter/foundation.dart';

class Time {
  const Time._(this._value);

  factory Time(String value) {
    assert(value != null);
    return Time._(
      _validateTime(value),
    );
  }

  factory Time.fromJson(dynamic json) => Time(json);

  factory Time.fromYaml(dynamic yaml) => yaml is String
      ? Time.fromJson(jsonDecode(jsonEncode(loadYaml(yaml))))
      : yaml is YamlMap
          ? Time.fromJson(jsonDecode(jsonEncode(yaml)))
          : null;

  final Either<String, String> _value;
  String get value => _value.fold((l) => l, (r) => r);
  bool get isValid => _value.isRight();

  String toString() => value;
  String toJson() => value;
  String toYaml() => value;

  bool operator ==(Object o) => identical(this, o)
      ? true
      : o is Time || o is String
          ? o == value
          : false;

  int get hashCode => value.hashCode;
}

Either<String, String> _validateTime(String value) =>
    RegExp(r'^([01][0-9]|2[0-3]):[0-5][0-9]:([0-5][0-9]|60)(\.[0-9]+)?$')
            .hasMatch(value)
        ? right(value)
        : left('FormatError: "$value" is not a Time, as defined by: '
            'https://www.hl7.org/fhir/datatypes.html#time');
